import {useState, useEffect} from 'react'
import {Stack, Typography, Button, FormControl, Grid, Box, Accordion, AccordionSummary, AccordionDetails, useTheme, TextField, Autocomplete} from '@mui/material'
import ExpandMoreIcon from '@mui/icons-material/ExpandMore'
import {axiosInstance} from '../providers/data'
import {BACKEND_URL} from '../lib/constants'
import {Link} from 'react-router'
import {TOKEN_KEY} from '../authProvider'

type Field<T> = {
  label: string
  data: keyof T
  render?: (value: any) => React.ReactNode
}

type ExtraFieldConfig = {
  type: 'number'
  label: string
  minValue: number
  maxValue: (linkedItems: any[]) => number
}

type LinkedItemsProps<T> = {
  parentId: string | number
  parentResource: string
  childResource: string
  fields: Field<T>[]
  title: string
  type: 'show' | 'edit'
  extraField?: ExtraFieldConfig
}

export const LinkedItems = <T extends {id: number; [key: string]: any}>({parentId, parentResource, childResource, fields, title, type}: LinkedItemsProps<T>) => {
  const [items, setItems] = useState<T[]>([])
  const [linkedItems, setLinkedItems] = useState<T[]>([])
  const [selectedItemId, setSelectedItemId] = useState<number | null>(null)
  const [pageNum, setPageNum] = useState<number>(1)
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [mediaOrder, setMediaOrder] = useState<number>(1)
  const theme = useTheme()

  useEffect(() => {
    if (parentId) {
      axiosInstance
        .get(`${BACKEND_URL}/${parentResource}/${parentId}/${childResource}`)
        .then((response) => {
          setLinkedItems(response?.data || [])
        })
        .catch(() => {
          setLinkedItems([])
        })
    }
  }, [parentId, parentResource, childResource])

  useEffect(() => {
    if (type === 'edit') {
      axiosInstance
        .get(`${BACKEND_URL}/${childResource}/`)
        .then((response) => {
          setItems(response?.data || [])
          setIsLoading(false)
        })
        .catch(() => {
          setItems([])
          setIsLoading(false)
        })
    } else {
      setIsLoading(false)
    }
  }, [childResource, type])

  useEffect(() => {
    if (childResource === 'article' && parentResource === 'sight') {
      setPageNum(linkedItems.length + 1)
    }
  }, [linkedItems, childResource, parentResource])

  const availableItems = items.filter((item) => !linkedItems.some((linked) => linked.id === item.id))

  const linkItem = () => {
    if (selectedItemId !== null) {
      const requestData =
        childResource === 'article'
          ? {
              [`${childResource}_id`]: selectedItemId,
              page_num: pageNum,
            }
          : childResource === 'media'
          ? {
              [`${childResource}_id`]: selectedItemId,
              media_order: mediaOrder,
            }
          : {
              [`${childResource}_id`]: selectedItemId,
            }

      axiosInstance
        .post(`${BACKEND_URL}/${parentResource}/${parentId}/${childResource}`, requestData)
        .then(() => {
          axiosInstance.get(`${BACKEND_URL}/${parentResource}/${parentId}/${childResource}`).then((response) => {
            setLinkedItems(response?.data || [])
            setSelectedItemId(null)
            if (childResource === 'article') {
              setPageNum(pageNum + 1)
            }
          })
        })
        .catch((error) => {
          console.error('Error linking item:', error)
        })
    }
  }

  const deleteItem = (itemId: number) => {
    axiosInstance
      .delete(`${BACKEND_URL}/${parentResource}/${parentId}/${childResource}`, {
        data: {[`${childResource}_id`]: itemId},
      })
      .then(() => {
        setLinkedItems((prev) => prev.filter((item) => item.id !== itemId))
      })
      .catch((error) => {
        console.error('Error unlinking item:', error)
      })
  }

  return (
    <Accordion>
      <AccordionSummary
        expandIcon={<ExpandMoreIcon />}
        sx={{
          background: theme.palette.background.paper,
          borderBottom: `1px solid ${theme.palette.divider}`,
        }}
      >
        <Typography variant="subtitle1" fontWeight="bold">
          Привязанные {title}
        </Typography>
      </AccordionSummary>

      <AccordionDetails sx={{background: theme.palette.background.paper}}>
        <Stack gap={2}>
          <Grid container gap={1.25}>
            {isLoading ? (
              <Typography>Загрузка...</Typography>
            ) : linkedItems.length > 0 ? (
              linkedItems.map((item, index) => (
                <Box
                  component={Link}
                  to={`/${childResource}/show/${item.id}`}
                  key={index}
                  sx={{
                    marginTop: '8px',
                    padding: '14px',
                    borderRadius: 2,
                    border: `2px solid ${theme.palette.divider}`,
                    width: childResource === 'article' ? '100%' : 'auto',
                    textDecoration: 'none',
                    color: 'inherit',
                    display: 'block',
                    '&:hover': {
                      backgroundColor: theme.palette.action.hover,
                    },
                  }}
                >
                  <Stack gap={0.25}>
                    {childResource === 'media' && item.id && (
                      <img
                        src={`https://wn.krbl.ru/media/${item.id}/download?token=${localStorage.getItem(TOKEN_KEY)}`}
                        alt={String(item.media_name)}
                        style={{
                          width: '100%',
                          height: '120px',
                          objectFit: 'contain',
                          marginBottom: '8px',
                          borderRadius: 4,
                        }}
                      />
                    )}
                    {fields.map(({label, data, render}) => (
                      <Typography variant="body2" color="textSecondary" key={String(data)}>
                        <strong>{label}:</strong> {render ? render(item[data]) : item[data]}
                      </Typography>
                    ))}
                    {type === 'edit' && (
                      <Button
                        variant="outlined"
                        color="error"
                        size="small"
                        onClick={(e) => {
                          e.preventDefault()
                          deleteItem(item.id)
                        }}
                        sx={{mt: 1.5}}
                      >
                        Отвязать
                      </Button>
                    )}
                  </Stack>
                </Box>
              ))
            ) : (
              <Typography color="textSecondary">{title} не найдены</Typography>
            )}
          </Grid>

          {type === 'edit' && (
            <Stack gap={2}>
              <Typography variant="subtitle1">Добавить {title}</Typography>
              <Autocomplete
                fullWidth
                value={availableItems.find((item) => item.id === selectedItemId) || null}
                onChange={(_, newValue) => setSelectedItemId(newValue?.id || null)}
                options={availableItems}
                getOptionLabel={(item) => String(item[fields[0].data])}
                renderInput={(params) => <TextField {...params} label={`Выберите ${title}`} fullWidth />}
                isOptionEqualToValue={(option, value) => option.id === value?.id}
                filterOptions={(options, {inputValue}) => {
                  // return options.filter((option) => String(option[fields[0].data]).toLowerCase().includes(inputValue.toLowerCase()))
                  const searchWords = inputValue
                    .toLowerCase()
                    .split(' ')
                    .filter((word) => word.length > 0)
                  return options.filter((option) => {
                    const optionWords = String(option[fields[0].data]).toLowerCase().split(' ')
                    return searchWords.every((searchWord) => optionWords.some((word) => word.startsWith(searchWord)))
                  })
                }}
              />

              {childResource === 'article' && (
                <FormControl fullWidth>
                  <TextField
                    type="number"
                    label="Номер страницы"
                    name="page_num"
                    value={pageNum}
                    onChange={(e) => {
                      const newValue = Number(e.target.value)
                      const minValue = linkedItems.length + 1 // page number on articles lenght
                      setPageNum(newValue < minValue ? minValue : newValue)
                    }}
                    fullWidth
                    InputLabelProps={{shrink: true}}
                  />
                </FormControl>
              )}

              {childResource === 'media' && type === 'edit' && (
                <FormControl fullWidth>
                  <TextField
                    type="number"
                    label="Порядок отображения медиа"
                    value={mediaOrder}
                    onChange={(e) => {
                      const newValue = Number(e.target.value)
                      const maxValue = linkedItems.length + 1
                      const value = Math.max(1, Math.min(newValue, maxValue))
                      setMediaOrder(value)
                    }}
                    fullWidth
                    InputLabelProps={{shrink: true}}
                  />
                </FormControl>
              )}

              <Button variant="contained" onClick={linkItem} disabled={!selectedItemId}>
                Добавить
              </Button>
            </Stack>
          )}
        </Stack>
      </AccordionDetails>
    </Accordion>
  )
}
