import {Autocomplete, Box, TextField, FormControlLabel, Checkbox, Typography} from '@mui/material'
import {Edit, useAutocomplete} from '@refinedev/mui'
import {useForm} from '@refinedev/react-hook-form'
import {Controller} from 'react-hook-form'
import {useParams} from 'react-router'
import {LinkedItems} from '../../components/LinkedItems'
import {StationItem, VehicleItem, stationFields, vehicleFields} from './types'

export const RouteEdit = () => {
  const {
    saveButtonProps,
    register,
    control,
    formState: {errors},
  } = useForm({})

  const {id: routeId} = useParams<{id: string}>()

  const {autocompleteProps: carrierAutocompleteProps} = useAutocomplete({
    resource: 'carrier',
    onSearch: (value) => [
      {
        field: 'short_name',
        operator: 'contains',
        value,
      },
    ],
  })

  return (
    <Edit saveButtonProps={saveButtonProps}>
      <Box component="form" sx={{display: 'flex', flexDirection: 'column'}} autoComplete="off">
        <Controller
          control={control}
          name="carrier_id"
          rules={{required: 'Это поле является обязательным'}}
          defaultValue={null}
          render={({field}) => (
            <Autocomplete
              {...carrierAutocompleteProps}
              value={carrierAutocompleteProps.options.find((option) => option.id === field.value) || null}
              onChange={(_, value) => {
                field.onChange(value?.id || '')
              }}
              getOptionLabel={(item) => {
                return item ? item.short_name : ''
              }}
              isOptionEqualToValue={(option, value) => {
                return option.id === value?.id
              }}
              filterOptions={(options, {inputValue}) => {
                return options.filter((option) => option.short_name.toLowerCase().includes(inputValue.toLowerCase()))
              }}
              renderInput={(params) => <TextField {...params} label="Выберите перевозчика" margin="normal" variant="outlined" error={!!errors.carrier_id} helperText={(errors as any)?.carrier_id?.message} required />}
            />
          )}
        />

        <TextField
          {...register('route_number', {
            required: 'Это поле является обязательным',
            setValueAs: (value) => String(value),
          })}
          error={!!(errors as any)?.route_number}
          helperText={(errors as any)?.route_number?.message}
          margin="normal"
          fullWidth
          InputLabelProps={{shrink: true}}
          type="text"
          label={'Номер маршрута'}
          name="route_number"
        />
        <Controller
          name="route_direction" // boolean
          control={control}
          defaultValue={false}
          render={({field}: {field: any}) => <FormControlLabel label="Прямой маршрут?" control={<Checkbox {...field} checked={field.value} onChange={(e) => field.onChange(e.target.checked)} />} />}
        />

        <Typography variant="caption" color="textSecondary" sx={{mt: 0, mb: 1}}>
          (Прямой / Обратный)
        </Typography>

        <Controller
          name="path"
          control={control}
          defaultValue={[]}
          rules={{
            required: 'Это поле является обязательным',
            validate: (value: unknown) => {
              if (!Array.isArray(value)) return 'Неверный формат'
              if (!value.every((point: unknown) => Array.isArray(point) && point.length === 2)) {
                return 'Каждая точка должна быть массивом из двух координат'
              }
              if (!value.every((point: unknown[]) => point.every((coord: unknown) => !isNaN(Number(coord)) && typeof coord === 'number'))) {
                return 'Координаты должны быть числами'
              }
              return true
            },
          }}
          render={({field, fieldState: {error}}) => (
            <TextField
              {...field}
              value={Array.isArray(field.value) ? JSON.stringify(field.value) : ''}
              onChange={(e) => {
                try {
                  const parsed = JSON.parse(e.target.value)
                  field.onChange(parsed)
                } catch {
                  field.onChange([])
                }
              }}
              error={!!error}
              helperText={error?.message} // 'Формат: [[lat1,lon1], [lat2,lon2], ...]'
              margin="normal"
              fullWidth
              InputLabelProps={{shrink: true}}
              type="text"
              label={'Координаты маршрута'}
              placeholder="[[1.1, 2.2], [2.1, 4.5]]"
              sx={{
                marginBottom: 2,
              }}
            />
          )}
        />
      </Box>

      {routeId && (
        <>
          <LinkedItems<StationItem> type="edit" parentId={routeId} parentResource="route" childResource="station" fields={stationFields} title="станции" />

          <LinkedItems<VehicleItem> type="edit" parentId={routeId} parentResource="route" childResource="vehicle" fields={vehicleFields} title="транспортные средства" />
        </>
      )}
    </Edit>
  )
}
